/*
 * SPDX-FileCopyrightText: 2017~2017 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include "fcitxqtconfiguiplugin.h"

namespace fcitx {

FcitxQtConfigUIPlugin::FcitxQtConfigUIPlugin(QObject *parent)
    : QObject(parent) {}

FcitxQtConfigUIPlugin::~FcitxQtConfigUIPlugin() {}
} // namespace fcitx
